﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Speech.Synthesis;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace boyomityan
{
    /// <summary>
    /// フォーム
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            // 初期状態
            button_start.Enabled = true;
            button_end.Enabled = false;
        }

        /// <summary>
        /// 開始ボタン
        /// </summary>
        private void button_start_Click(object sender, EventArgs e)
        {
            clearTextLog();
            addTextLog("start ... ");

            button_start.Enabled = false;
            button_end.Enabled = true;

            prevString = getClipboardString();

            timer_clipboard.Start();
        }

        /// <summary>
        /// 終了ボタン
        /// </summary>
        private void button_end_Click(object sender, EventArgs e)
        {
            addTextLog("end ... ");
            timer_clipboard.Stop();

            button_start.Enabled = true;
            button_end.Enabled = false;
        }

        private bool isExecTimer = false;
        private string prevString = "";

        /// <summary>
        /// タイマ
        /// </summary>
        private void timer_clipboard_Tick(object sender, EventArgs e)
        {
            if (isExecTimer)
                return;
            isExecTimer = true;

            try
            {
                // クリップボードから文字列を取得
                var str = getClipboardString();
                if (prevString != str)
                {
                    // 喋らせる
                    addTextLog(str);
                    speechString(str,
                                (int)numericUpDown_volume.Value,
                                (int)numericUpDown_speed.Value);
                    prevString = str;
                }
            }
            catch
            {
            }
            finally
            {
                isExecTimer = false;
            }
        }

        /// <summary>
        /// ログをクリアする
        /// </summary>
        private void clearTextLog()
        {
            textBox_log.Text = "";
        }

        /// <summary>
        /// ログを追加する
        /// </summary>
        /// <param name="str">文字列</param>
        private void addTextLog(string str)
        {
            textBox_log.Text = str + "\r\n" + textBox_log.Text;
        }

        /// <summary>
        /// クリップボードからテキストを取得する
        /// </summary>
        /// <returns>文字列</returns>
        private string getClipboardString()
        {
            try
            {
                var data = Clipboard.GetDataObject();
                if (data.GetDataPresent(DataFormats.Text))
                {
                    var str = (string)Clipboard.GetText();
                    return str;
                }
                else
                {
                    return "";
                }
            }
            catch
            {
                return "";
            }
        }

        private SpeechSynthesizer speechSynthesizer = new SpeechSynthesizer();

        /// <summary>
        /// 文字列を喋らせる
        /// </summary>
        /// <param name="str">文字列</param>
        /// <param name="volume">音量</param>
        /// <param name="rate">速度</param>
        private void speechString(string str, int volume, int rate)
        {
            try
            {
                speechSynthesizer.Volume = volume;
                speechSynthesizer.Rate = rate;
                speechSynthesizer.Speak(str);       // 同期
                //speechSynthesizer.SpeakAsync(str);  // 非同期
            }
            catch
            {
            }
        }
    }
}
